@@ -67,7 +67,7 @@ def handle_input_parameter(name, par, default_min=None, default_max=None):
             min_value = par[0]
             max_value = par[1]
 
-            return ipars.FreeInstrumentParameter(name, min_value, max_value)
+            return ipars.RelativeFreeInstrumentParameter(name, min_value, max_value)
 
         else:
             # Assume a fixed value is given
@@ -78,7 +78,7 @@ def handle_input_parameter(name, par, default_min=None, default_max=None):
     
     else:
         # If nothing is specified, use default range (can both be None, None for length / start_point)
-        return ipars.FreeInstrumentParameter(name, default_min, default_max)
+        return ipars.RelativeFreeInstrumentParameter(name, default_min, default_max)
 
 
 class Element:
@@ -117,15 +117,15 @@ class Element:
 
         self.name = name
 
+        if isinstance(length, ipars.InstrumentParameter):
+            raise ValueError("length parameter not allowed to be custom parameter.")
+
+        if isinstance(start_point, ipars.InstrumentParameter):
+            raise ValueError("start_point parameter not allowed to be custom parameter.")
+
         self.length = handle_input_parameter(name + "_length", length)
         self.start_point = handle_input_parameter(name + "_start_point", start_point)
         
-        if isinstance(self.length, ipars.DependentInstrumentParameter):
-            raise ValueError("length parameter not allowed to be dependent parameter.")
-        
-        if isinstance(self.start_point, ipars.DependentInstrumentParameter):
-            raise ValueError("start_point parameter not allowed to be dependent parameter.")
-        
         self.start_point_parameter = None
         self.next_start_point_parameter = None
 
@@ -321,6 +321,38 @@ class GuideElement(Element):
     def calculate_length(self):
         self.element_length = self.next_start_point_parameter.get_value() - self.start_point_parameter.get_value()
 
+    def write_to_log_base(self, file):
+        """
+        Base method for writing basic information to guide log file, not to be overwritten
+        """
+
+        file.write("\nElement " + self.__class__.__name__ + " " + self.name + "\n")
+        self.write_parameter(file, "start_point", self.start_point_parameter, "start_point")
+        self.write_parameter(file, "next_start_point", self.next_start_point_parameter, "start_point")
+        self.write_parameter(file, "start_width", self.start_width, "horizontal")
+        self.write_parameter(file, "start_height", self.start_height, "vertical")
+        self.write_parameter(file, "end_width", self.end_width, "horizontal")
+        self.write_parameter(file, "end_height", self.end_height, "vertical")
+
+    def write_parameter(self, file, log_name, parameter, type_string=None):
+        """
+        Method for writing to log file in order to keep consistency and do error checks
+        """
+        if type_string not in ("horizontal", "vertical", "start_point", None):
+            raise RuntimeError("type_string needs to be horizontal, vertical, start_point or None")
+
+        if type_string is None:
+            type_string = ""
+
+        file.write(" " + log_name.ljust(29) + type_string.ljust(15) + parameter.name.ljust(50) + "\n")
+
+    def write_to_log(self, file):
+        """
+        Template function which should be overwritten for guide elements to add more details as necessary
+        """
+
+        self.write_to_log_base(file)
+
     def __repr__(self):
         """
         Provides a basic description of the guide element
@@ -331,6 +363,9 @@ class GuideElement(Element):
         string += self.length.__repr__() + "\n"
         string += self.start_point.__repr__() + "\n"
         string += self.start_width.__repr__() + "\n"
-        string += self.start_height.__repr__()
+        string += self.start_height.__repr__() + "\n"
+
+        string += self.start_point_parameter.__repr__() + "\n"
+        string += self.next_start_point_parameter.__repr__()
 
         return string
