@@ -46,11 +46,12 @@ class Guide:
         name : str
             Optional name of the guide
         """
-        self.name = name
-        self.generate_name()
-
         self.guide_elements = []
 
+        self.name = name
+        self.auto_name = None
+        self.generate_name()
+
         self.original_guide_elements = None
 
         self.constraints = []
@@ -73,7 +74,11 @@ class Guide:
         """
         Generates a name for this guide using first letter of each element
         """
-        if self.name is None:
+
+        if self.auto_name is None:
+            self.auto_name = self.name is None
+
+        if self.auto_name:
             self.name = "Guide_"
             for element in self.guide_elements:
                 first_letter = type(element).__name__[0]
@@ -123,10 +128,14 @@ class Guide:
         guide_element : GuideElement
             New element added to the end of the guide
         """
+
+        # todo: check element name not already in use
+
         if self.current_owner is not None:
             guide_element.set_owner(self.current_owner)
 
         self.guide_elements.append(guide_element)
+        self.generate_name()
 
     def __iadd__(self, guide_element):
         """
@@ -161,6 +170,7 @@ class Guide:
         if self.current_owner is not None:
             guide_element.set_owner(self.current_owner)
 
+        # todo: check element name not already in use
         self.guide_elements = [guide_element] + self.guide_elements
 
     def add_constraint(self, constraint):
@@ -306,6 +316,14 @@ class Guide:
         moderator.add_to_instrument(instrument, instrument_parameters, guide)
         """
 
+    def write_log_file(self, filename):
+        # Start file
+        with open(filename, "w") as file:
+            file.write("Guide log file from python guide_bot\n")
+
+            for element in self.guide_elements:
+                element.write_to_log(file)
+
     def __repr__(self):
         """
         Provides a string describing the Guide object
@@ -323,7 +341,7 @@ class Guide:
         return string
 
 
-class guide_bot_run:
+class Project:
     """
     Main logic for performing a guide_bot run with any number of guides
 
@@ -333,24 +351,24 @@ class guide_bot_run:
     the guide optimization, and a moderator object describing the neutron
     source including basic restrictions. New Guide objects can be
     generated by the new_guide method, which can then be customized but are
-    still connected to the guide_bot_run instance. Call the run method to
-    perform the optimization.
+    still connected to the Project instance. Call the run method to perform
+    the optimization.
     """
     def __init__(self, name, sample, moderator, analysis_moderators=None, settings=None):
         """
-        Create a new guide_bot_run from sample and moderator specifications
+        Create a new Project from sample and moderator specifications
 
-        The guide_bot_run object contains the main logic for performing a
-        guide_bot run and serves as part of the user interface. New guide
-        objects can be generated with the new_guide method, and these can then
-        have guide elements added to them to describe the guide geometry. The
-        run needs a name which will be used to identify the generated folders.
+        The Project object contains the main logic for performing a guide_bot
+        run and serves as part of the user interface. New guide objects can be
+        generated with the new_guide method, and these can then have guide
+        elements added to them to describe the guide geometry. The project
+        needs a name which will be used to identify the generated folders.
 
         Parameters
         ----------
 
         name : str
-            Name of this run
+            Name of this project
 
         sample : Sample object
             Sample object describing the figure of merit for optimization
@@ -395,13 +413,13 @@ class guide_bot_run:
 
     def add_guide(self, guide):
         """
-        Adds a Guide object to the guide_bot_run for subsequent optimization
+        Adds a Guide object to the Project for subsequent optimization
 
         Parameters
         ----------
 
         guide : Guide
-            Guide object to be added to the guide_bot_run
+            Guide object to be added to the Project
         """
         for element in guide.guide_elements:
             element.set_owner("User")
@@ -414,7 +432,7 @@ class guide_bot_run:
 
         The returned guide object can be expanded by the user to describe the
         desired guide geometry, and does not need to be added to the
-        guide_bot_run object as it is already registered.
+        Project object as it is already registered.
 
         Returns
         -------
@@ -430,7 +448,7 @@ class guide_bot_run:
 
     def __repr__(self):
         """
-        Prints status of guide_bot_run object
+        Prints status of Projectrun object
 
         Returns
         -------
@@ -439,7 +457,7 @@ class guide_bot_run:
             String with description of status
         """
 
-        string = "guide_bot_run named '" + self.name + "'\n"
+        string = "guide_bot Project named '" + self.name + "'\n"
 
         string += "Included guides: \n"
         for guide in self.guides:
@@ -513,6 +531,9 @@ class guide_bot_run:
         if "minfunc" not in self.settings:
             self.settings["minfunc"] = 1E-8
 
+        if "logfile" not in self.settings:
+            self.settings["logfile"] = False
+
         self.settings["optimized_monitor"] = "fom.dat"
         self.settings["foldername"] = "optimization_data"
 
@@ -524,13 +545,13 @@ class guide_bot_run:
         dill.dump(package, outfile)
         outfile.close()
 
-    def run(self, cluster=None):
+    def write(self, cluster=None):
         """
         Saves optimization job packages for all input configurations
 
-        The run method calls the package_single method for each guide and scans
+        The write method calls the package_single method for each guide and scans
         over all the input configurations from moderator and sample.
-        Writes an overview yaml file describing the run which will aid in
+        Writes an overview yaml file describing the project which will aid in
         plotting an overview of the results.
         """
         scan = rpars.InputConfigurationIterator(self.sample, self.moderator)
@@ -539,8 +560,9 @@ class guide_bot_run:
         try:
             os.mkdir(self.name)
         except OSError:
-            raise RuntimeError("Could not create folder for guide_bot run!")
-        self.base_folder = os.path.join(os.getcwd(), self.name)
+            raise RuntimeError("Could not create folder for guide_bot project!")
+        self.original_wd = os.getcwd()
+        self.base_folder = os.path.join(self.original_wd, self.name)
 
         if cluster is not None:
             cluster.set_project_path(self.base_folder)
@@ -607,6 +629,8 @@ class guide_bot_run:
         with open("run_overview.yaml", 'w') as yaml_file:
             yaml.dump(overview, yaml_file, default_flow_style=False)
 
+        os.chdir(self.original_wd)
+
 
 
 
