@@ -3,50 +3,15 @@ import pyswarm
 
 import numpy as np
 
+from guide_bot.parameters.instrument_parameters import FreeInstrumentParameter
+from guide_bot.parameters.instrument_parameters import RelativeFreeInstrumentParameter
+from guide_bot.parameters.instrument_parameters import FixedInstrumentParameter
+from guide_bot.parameters.instrument_parameters import DependentInstrumentParameter
+
+from guide_bot.logging import guide_log
+
 from mcstasscript.interface import instr, functions, plotter
 
-
-class RunInfo:
-    """
-    Container for information from a single optimization step
-
-    Saves the optimizer state and figure of merit from a step in the
-    optimization.
-    """
-    def __init__(self, optimized_x, fom):
-        self.optimized_x = optimized_x
-        self.fom = fom
-
-
-class History:
-    """
-    Container for RunInfo instances for each step in the optimization
-
-    Keeps a history of optimizer states and figure of merits organized in
-    RunInfo instances.
-    """
-    def __init__(self):
-        """
-        Starts a history container
-        """
-        self.histories = []
-    
-    def add_run_info(self, optimized_x, fom):
-        """
-        Appends information to the history
-
-        Parameters
-        ----------
-
-        optimized_x : list
-            Optimizer state, list of values for each free parameter
-
-        fom : float
-            Figure of merit attained by optimized_x state
-        """
-        self.histories.append(RunInfo(optimized_x, fom))
-
-
 def optimizer_func(optimizer_x, *args, **kwargs):
     """
     Optimizer function for swarm algorithm, calculates fom with simulation
@@ -81,17 +46,23 @@ def optimizer_func(optimizer_x, *args, **kwargs):
     instrument = kwargs["instrument"]
     instr_parameters = kwargs["instrument_parameters"]
     settings = kwargs["settings"]
+    logfile = kwargs["logfile"]
 
     instr_parameter_input = instr_parameters.extract_instrument_parameters(optimizer_x)
-    print(instr_parameters)
+
+    if not settings["logfile"]:
+        print(instr_parameters)
 
     constraint_values = instr_parameters.evaluate_constraints()
 
-    if np.min(constraint_values) < 0.0:
+    if len(constraint_values) > 0 and np.min(constraint_values) < 0.0:
         print(" -- Returned 0 for fom due to unfulfilled constraints! -- ")
+        if settings["logfile"]:
+            logfile.write_parameter_line(0.0, instr_parameter_input)
         return 0.0
 
-    print("Running with \n", instr_parameter_input)
+    if not settings["logfile"]:
+        print("Running with \n", instr_parameter_input)
 
     sim_data = instrument.run_full_instrument(foldername=settings["foldername"],
                                               increment_folder_name=True, force_compile=False,
@@ -104,6 +75,9 @@ def optimizer_func(optimizer_x, *args, **kwargs):
     fom = -np.sum(np.sum(optimizer_data.Intensity))
     print("  fom =", fom)
 
+    if settings["logfile"]:
+        logfile.write_parameter_line(fom, instr_parameter_input)
+
     if not isinstance(fom, float):
         print("Warning! FOM not a float!")
     
@@ -111,6 +85,7 @@ def optimizer_func(optimizer_x, *args, **kwargs):
     
     return fom
 
+
 def remove_data_folder(path):
     """
     Safe remove function that will only delete folders that do not contain
@@ -142,7 +117,7 @@ def optimizer_f_ieqcons(optimizer_x, *args, **kwargs):
     return instr_parameters.evaluate_constraints(optimizer_x)
 
 
-def run_optimizer(instrument, my_parameters, settings):
+def run_optimizer(instrument, my_parameters, settings, scan_name):
     """
     Performs optimization of given instrument with parameters and settings
 
@@ -176,21 +151,28 @@ def run_optimizer(instrument, my_parameters, settings):
     lb = my_parameters.get_lower_bounds()
     ub = my_parameters.get_upper_bounds()
 
-    my_history = History()
+    my_history = guide_log.History()
+
+    if settings["logfile"]:
+        guide_name = os.path.split(settings["foldername"])[0]
+        logfile = guide_log.LogFile(guide_name + ".log")
+        logfile.write_header(my_parameters, scan_name)
+    else:
+        logfile = None
 
     kw_package = {"instrument": instrument, "instrument_parameters": my_parameters,
-                  "settings": settings, "history": my_history}
+                  "settings": settings, "history": my_history, "logfile": logfile}
 
     xopt, fopt = pyswarm.pso(optimizer_func, lb, ub, f_ieqcons=optimizer_f_ieqcons, kwargs=kw_package,
                              swarmsize=settings["swarmsize"], omega=settings["omega"], phip=settings["phip"],
                              phig=settings["phig"], maxiter=settings["maxiter"], minstep=settings["minstep"],
                              minfunc=settings["minfunc"], debug=False)
 
-    print("-"*50, "done", "-"*50)
-    print(xopt)
-    print(fopt)
+    print("-"*30, "done", "-"*50)
+    print("best x:", xopt)
+    print("best fom: ", fopt)
 
-    print("-"*50, "showing history", "-"*50)
+    print("-"*30, "showing history", "-"*50)
     lowest_fom = 1
     lowest_fom_pars = None
     for history in my_history.histories:
@@ -200,7 +182,7 @@ def run_optimizer(instrument, my_parameters, settings):
             lowest_fom = history.fom
             lowest_fom_pars = instrument_pars
 
-    print("-"*50, "best from history", "-"*50)
+    print("-"*30, "best from history", "-"*50)
     print("Best fom: ", lowest_fom)
     print("From parameters", lowest_fom_pars)
 
