@@ -1,3 +1,5 @@
+import math
+
 import numpy as np
 import yaml
 
@@ -38,7 +40,8 @@ class Sample:
     def __init__(self, width, height,
                  div_horizontal, div_vertical,
                  min_wavelength, max_wavelength,
-                 instrument_length, sample_guide_distance):
+                 instrument_length, sample_guide_distance,
+                 calculate_guide_end_dimensions=True):
         """
         Description of figure of merit for guide optimization
 
@@ -73,9 +76,13 @@ class Sample:
 
         sample_guide_distance : float or list of floats for scan
             Distance from guide end to sample in [m]
+
+        calculate_guide_end_dimensions: bool or list of bools for scan (default True)
+            If true, the guides end dimensions are calculated from sample requirements
         """
         self.parameters = Parameters()
 
+        # Required
         self.parameters.add("width", width, unit="m")
         self.parameters.add("height", height, unit="m")
         self.parameters.add("div_horizontal", div_horizontal, unit="deg")
@@ -85,6 +92,9 @@ class Sample:
         self.parameters.add("instrument_length", instrument_length, unit="m")
         self.parameters.add("sample_guide_distance", sample_guide_distance, unit="m")
 
+        # Optional (has default value)
+        self.parameters.add("calculate_guide_end_dimensions", calculate_guide_end_dimensions, unit="bool")
+
         # list of PlotInfo objects that will be applied to control plotting with McStasScript
         self.plot_info = PlotInfoContainer()
 
@@ -185,8 +195,24 @@ class Sample:
         instrument_parameters : InstrumentParameterContainer
             Parameter container where parameters can be added for optimization
         """
+
+        # Use minimalist principle to calculate guide end dimensions
+        width = self["width"] + 2 * math.tan(self["div_horizontal"] * math.pi / 180.0) * self["sample_guide_distance"]
+        height = self["height"] + 2 * math.tan(self["div_vertical"] * math.pi / 180.0) * self["sample_guide_distance"]
+
+
+        if self["calculate_guide_end_dimensions"]:
+            # If end dimensions should be calculated, use exact values
+            width_input = width
+            height_input = height
+        else:
+            # If end dimensions should be optimized, use calculated values to set reasonable limits
+            width_input = [0.5*width, 1.5*width]
+            height_input = [0.5*height, 1.5*height]
+
         end_gap = Element_gap.Gap(name="guide_sample_gap",
-                                  length=self["sample_guide_distance"])
+                                  length=self["sample_guide_distance"],
+                                  start_width=width_input, start_height=height_input)
 
         guide.add_guide_element(end_gap)
 
